


module Buildwise
	class App
		module BuildsHelper




			def find_build(id)
        if defined?(SERVER_DIGEST)
          @build = Build.where(:server_digest => SERVER_DIGEST, :id => id).first
        else 
          @build = Build.find_by_id(id)
        end
				@project = @build.project if @build
				return @build
			end

			def distributed_build_in_progress
				@test_files = TestFile.where(:build_id => @build.id).includes(:test_cases).order("priority DESC, filename ASC")
			end

			def distributed_build_complete
				@test_files ||= TestFile.where(:build_id => @build.id).includes(:test_cases).order("assigned_at ASC")
			end

			def quick_build_in_progress
				@ui_test_files = YAML.load(@build.acceptance_test_options)[:ui_test_file_list].split(",") rescue []
				@refresh_interval = 10
				logger.debug("Refresh UI Test report in progress: Build #{@build.id}: #{@build.stage}")

				if @build.is_ui_test_stage?
					ui_test_report_dir = @build.determine_ui_test_report_dir
					ui_test_dir = artifact_ui_test_dir = File.join(@build.artifacts_dir, "ui-tests")
					logger.debug("Parsing test results, folder => |#{ui_test_report_dir}| and test_dir => #{ui_test_dir}")
					if ui_test_report_dir

						ui_test_framework = @build.project.config.builder_ui_test_framework || @build.test_framework
						if ui_test_framework.nil? || ui_test_framework.empty?
							@build.test_framework = @build.determine_test_syntax_framework(ui_test_dir)        

						else
							@build.test_framework = ui_test_framework
						end


						begin
							case  @build.test_framework               
							when "Cucumber"
								@test_files = @build.parse_feature_results(ui_test_dir, ui_test_report_dir, :complete => false, :file_pattern => /^TEST.*\.xml$/i)
                  
							when "Mocha"
								@test_files = @build.parse_spec_results(ui_test_dir, ui_test_report_dir, :complete => false, :file_pattern => /^*\.xml$/i)


                @test_files.each do |tf|
                  if tf.new_record?
                    tf.filename = tf.filename.sub /\.[^\.]+$/, '.js'
                  end
                end
                
							when "unittest" # python       
								@test_files = @build.parse_spec_results(ui_test_dir, ui_test_report_dir, :complete => false, :file_pattern =>  /^*\.xml$/i)
                

                @test_files.each do |tf|
                  if tf.new_record?
                    tf.filename = tf.filename.sub /\.[^\.]+$/, '.py'
                  end
                end
                
							when "RSpec" # default to RSpec
								if File.exists?(File.join(ui_test_dir, "spec"))
									ui_test_dir = File.join(ui_test_dir, "spec")
								end

								@test_files = @build.parse_spec_results(ui_test_dir, ui_test_report_dir, :complete => false, :file_pattern => /^[SPEC|TEST].*\.xml$/i)
								
							else # unkown       

								@test_files = @build.parse_spec_results(ui_test_dir, ui_test_report_dir, :complete => false, :file_pattern =>  /^*\.xml$/i)
															
              end


						rescue TypeError => te
							puts "[ERROR] failed to parse tests: #{te}"

						end
					end
				end
			end

			def quick_build_complete
				@test_files = TestFile.where(:build_id => @build.id)
			end
      
      def is_server_test_execution_capable?(a_build)
        return false if a_build.nil? || a_build.class != Build || a_build.project.nil?
        @project = a_build.project
        

        if @build.is_parallel
          return false
        end
        
        is_rspec_project = @project.config.builder_ui_test_framework == "RSpec"
        is_mocha_project = @project.config.builder_ui_test_framework == "Mocha"
        is_pytest_project = @project.config.builder_ui_test_framework == "unittest" # python
        
        is_ui_test_project_dir_configured =  (@project.config.ui_tests_local_working_dir && Dir.exist?(@project.config.ui_tests_local_working_dir) )  ||  (@project.config.ui_tests_dir && Dir.exist?(@project.config.ui_tests_dir) ) 
        
        @capable_server_test_execution = (is_rspec_project || is_mocha_project || is_pytest_project) && is_ui_test_project_dir_configured
      end
			
      




      def detect_hanging_test_execution(a_build, max_time = 600)
        return if a_build.nil?
        if a_build.incomplete?
          agent_allocations = a_build.current_agent_allocations
          agent_allocations.sort!{|a,b| a.assigned_at <=> b.assigned_at }
          
          oldest_allocation = agent_allocations.first
          if (oldest_allocation)

            test_file_id = oldest_allocation.test_file_id
            @test_file = TestFile.find(test_file_id)
            execution_time = Time.now - oldest_allocation.assigned_at

            if @test_file && @test_file.build_id == a_build.id && execution_time > max_time
              logger.info("Detect a long running test execution on agent '#{oldest_allocation.agent_name}', #{execution_time}s (over #{max_time}), cancel it.")
              a_build.unassign_test(@test_file)                          
            end
          end
        end        
      end
      	
      

      def execute_rake_task_by_child_process(rake_command_arguments, envs = {})      
        require "childprocess"
         
        process = ChildProcess.build("rake", rake_command_arguments)
        process.detach = false  
        temp_file =  Tempfile.new("rake-output")     
        process.io.stdout = temp_file
        process.io.stderr = temp_file

        envs.each do |key, val|
          process.environment[key] = val                         
        end
                
        process.start        
        begin
          process.poll_for_exit(2 * 60) # 2 minutes
        rescue ChildProcess::TimeoutError
          process.stop # tries increasingly harsher methods to kill the process.
        end
        
        return File.read(temp_file)
      end
      
      def execute_rake_task(rake_command_arguments, work_dir = nil,  envs = {})   
        require 'open3'
        cmd = "rake #{rake_command_arguments}"
        saved_work_dir = File.expand_path('.')
        begin 
        FileUtils.chdir(work_dir) if work_dir
          puts("About to run command: #{cmd} in #{work_dir}")      
          stdout, stderr, status = Open3.capture3(cmd)          

          puts("OK")  
          return {:stdout => stdout, :stderr => stderr, :status => status}    
        ensure 
          FileUtils.chdir(saved_work_dir)
        end
      end
      

		end

		helpers BuildsHelper
	end
end